#include <p.h>

/* Path Integral Monte Carlo code developed by A. Balaz 
   (antun@ipb.ac.rs) for papers:
         
   "Systematic Speedup in the Convergence of Path Integrals"
   by A. Bogojevic, A. Balaz, and A. Belic
   Phys. Rev. Lett. 94 (2005) 180403

   "Fast Converging Path Integrals for Time-Dependent Potentials"
   by A. Balaz, I. Vidanovic, A. Bogojevic, A, Pelster
   arXiv:0912.2743

   Address: Institute of Physics, Belgrade, Serbia
   Scientific Computing Laboratory, http://www.scl.rs/speedup/

   Public use and modification of this code is allowed providing 
   the above paper is properly acknowledged. The author would be 
   grateful for all information and/or comments regarding the use   
   of the code.  
*/

/* USER CONFIGURABLE SECTION:

   1) Defining the level parameter p (1 <= p <= 7). 
   2) Supplying the C function for the evaluation of the potential V
      and its parameters.
*/

/* 1) Defining the level parameter p (1 <= p <= 7)

      The level parameter p corresponds to the function Vp()
      (improved potential) which will be compiled. You need to adjust
      the following preprocessor variable. The value below is for p = 4.
*/

#define _p 4

/* 2) Supplying the C function for the evaluation of the potential V
      and its parameters.

      Function V0(x) provides the value of the potential parameters,
      used later to calculate the effective potential.

      Parameters of the potential are par[0] ..., as entered by
      you on the command line. $\phi^4$ example is given below.
*/

void V0(double x, double t) /* Example of the user supplied function V0() for the pure quartic rescaled with the Grosche factor sqrt(t*t+1). */
{
   xs[1] = x;
   xs[2] = x * x;
   xs[3] = xs[2] * xs[1];
   xs[4] = xs[3] * xs[1];
   ts[1] = t;
   ftinv[1] = 1 / (t * t + 1);
   ftinv[2] = ftinv[1] * ftinv[1];
   ftinv[3] = ftinv[2] * ftinv[1];

   return;
}

/* END OF THE USER CONFIGURABLE SECTION! */


/* Function func(q[]) returns exp(- action + norm), where the variable action is
   the action for the given path (q[0], q[step], ..., q[N]) plus the exponent of
   the Gaussian probability density function (*distrexp), and norm is the normalization
   (*norm) of the path integral (which also includes the normalization of Gaussians).
   Array *norm is defined in main(), array *distrexp in distr().
   The improved potential Vp() for every pair of points (q[i], q[i + step])
   is evaluated at the mid-point qbar.
*/

double func(double *q, int s, int n)
{
   void V0(double, double); /* The user supplied function that returns the unimproved potential
                               and its derivatives. */
   double Vp(void); /* The function that, for a given p, returns the improved potential. */
   double tbar; /* Time mid-point. */

   int i; /* Loop variable for the loop over the time steps. */
   int j; /* Loop variable for the loop defining deltas array. */

   action = 0; /* Initialiation of the action. */
   Nmax = two[s]; /* The maximal number of time steps. */
   Nc = two[n]; /* The current number of time steps. */
   step = two[s - n]; /* Index distance of path nodes at current bisection level. */
   epss[1] = T / Nc; /* Current time step and related variables. */
   epsinv = 1. / epss[1];
   tbar = ta - 0.5 * epss[1];
   for(i = 2; i < _p; i++) epss[i] = epss[1] * epss[i - 1];
   gs[1] = par[0];

   for(i = 0; i < Nmax; i += step) /* Loop over path nodes at current bisection level. */
   {
      tbar += epss[1];
      qbar = (q[i + step] + q[i]) * 0.5; /* Mid-point for nearest nodes. */
      deltas[1] = 0.5 * (q[i + step] - q[i]); /* Half-distance between nearest nodes. */
      deltas[2] = deltas[1] * deltas[1];
      for(j = 3; j < 2 * _p - 1; j++) deltas[j] = deltas[1] * deltas[j - 1];
      V0(qbar, tbar); /* Initialization of unimproved potential and its derivatives at qbar. */
      action += 2 * deltas[2] * epsinv + epss[1] * Vp() - distrexp[i]; /* Accumulation of the action minus
                                                                     the exponent of the Gaussian. */
   }

   return exp(- action + norm[n]); /* norm[n] includes the path integral norm for a
                                      given number of time steps 2^n and normalization of Gaussians
                                      used as probability density functions. */
}

/* For a chosen p, the function Vp() returns the value of the improved potential. */

#if _p == 1
double Vp(void)
{
   return (ftinv[3]*gs[1]*xs[4])/24;
}
#endif

#if _p == 2
double Vp(void)
{
   return (deltas[2]*ftinv[3]*gs[1]*xs[2])/12 + (epss[1]*ftinv[3]*gs[1]*xs[2])/24 + (ftinv[3]*gs[1]*xs[4])/24;           
}
#endif

#if _p == 3
double Vp(void)
{
   xs[5] = xs[4] * xs[1];
   xs[6] = xs[5] * xs[1];
   gs[2] = gs[1] * gs[1];
   ts[2] = ts[1] * ts[1];
   ftinv[4] = ftinv[3] * ftinv[1];
   ftinv[5] = ftinv[4] * ftinv[1];
   ftinv[6] = ftinv[5] * ftinv[1];
   return (deltas[4]*ftinv[3]*gs[1])/120 + (deltas[2]*epss[1]*ftinv[3]*gs[1])/120 + (epss[2]*ftinv[3]*gs[1])/240 + (deltas[2]*ftinv[3]*gs[1]*xs[2])/12 + 
 (epss[1]*ftinv[3]*gs[1]*xs[2])/24 - (deltas[1]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[3])/6 + (ftinv[3]*gs[1]*xs[4])/24 - (epss[2]*ftinv[4]*gs[1]*xs[4])/96 + 
 (epss[2]*ftinv[5]*gs[1]*ts[2]*xs[4])/12 - (epss[2]*ftinv[6]*gs[2]*xs[6])/864;
}
#endif

#if _p == 4
double Vp(void)
{
   xs[5] = xs[4] * xs[1];
   xs[6] = xs[5] * xs[1];
   gs[2] = gs[1] * gs[1];
   ts[2] = ts[1] * ts[1];
   ftinv[4] = ftinv[3] * ftinv[1];
   ftinv[5] = ftinv[4] * ftinv[1];
   ftinv[6] = ftinv[5] * ftinv[1];
   return (deltas[4]*ftinv[3]*gs[1])/120 + (deltas[2]*epss[1]*ftinv[3]*gs[1])/120 + (epss[2]*ftinv[3]*gs[1])/240 - 
 (deltas[3]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[1])/10 - (deltas[1]*epss[2]*ftinv[4]*gs[1]*ts[1]*xs[1])/20 + (deltas[2]*ftinv[3]*gs[1]*xs[2])/12 + 
 (epss[1]*ftinv[3]*gs[1]*xs[2])/24 - (3*deltas[2]*epss[2]*ftinv[4]*gs[1]*xs[2])/80 - (epss[3]*ftinv[4]*gs[1]*xs[2])/160 + 
 (3*deltas[2]*epss[2]*ftinv[5]*gs[1]*ts[2]*xs[2])/10 + (epss[3]*ftinv[5]*gs[1]*ts[2]*xs[2])/20 - (deltas[1]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[3])/6 + 
 (ftinv[3]*gs[1]*xs[4])/24 - (epss[2]*ftinv[4]*gs[1]*xs[4])/96 - (deltas[2]*epss[2]*ftinv[6]*gs[2]*xs[4])/480 - (epss[3]*ftinv[6]*gs[2]*xs[4])/480 + 
 (epss[2]*ftinv[5]*gs[1]*ts[2]*xs[4])/12 - (epss[2]*ftinv[6]*gs[2]*xs[6])/864;
}
#endif

#if _p == 5
double Vp(void)
{
   xs[5] = xs[4] * xs[1];
   xs[6] = xs[5] * xs[1];
   xs[7] = xs[6] * xs[1];
   xs[8] = xs[7] * xs[1];
   gs[2] = gs[1] * gs[1];
   gs[3] = gs[2] * gs[1];
   ts[2] = ts[1] * ts[1];
   ts[3] = ts[2] * ts[1];
   ts[4] = ts[3] * ts[1];
   ftinv[4] = ftinv[3] * ftinv[1];
   ftinv[5] = ftinv[4] * ftinv[1];
   ftinv[6] = ftinv[5] * ftinv[1];
   ftinv[7] = ftinv[6] * ftinv[1];
   ftinv[8] = ftinv[7] * ftinv[1];
   ftinv[9] = ftinv[8] * ftinv[1];
   return (deltas[4]*ftinv[3]*gs[1])/120 + (deltas[2]*epss[1]*ftinv[3]*gs[1])/120 + (epss[2]*ftinv[3]*gs[1])/240 - (deltas[4]*epss[2]*ftinv[4]*gs[1])/224 - 
 (3*deltas[2]*epss[3]*ftinv[4]*gs[1])/1120 - (epss[4]*ftinv[4]*gs[1])/2240 + (deltas[4]*epss[2]*ftinv[5]*gs[1]*ts[2])/28 + 
 (3*deltas[2]*epss[3]*ftinv[5]*gs[1]*ts[2])/140 + (epss[4]*ftinv[5]*gs[1]*ts[2])/280 - (deltas[3]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[1])/10 - 
 (deltas[1]*epss[2]*ftinv[4]*gs[1]*ts[1]*xs[1])/20 + (deltas[2]*ftinv[3]*gs[1]*xs[2])/12 + (epss[1]*ftinv[3]*gs[1]*xs[2])/24 - 
 (3*deltas[2]*epss[2]*ftinv[4]*gs[1]*xs[2])/80 - (epss[3]*ftinv[4]*gs[1]*xs[2])/160 - (deltas[4]*epss[2]*ftinv[6]*gs[2]*xs[2])/1440 - 
 (deltas[2]*epss[3]*ftinv[6]*gs[2]*xs[2])/720 - (epss[4]*ftinv[6]*gs[2]*xs[2])/1152 + (3*deltas[2]*epss[2]*ftinv[5]*gs[1]*ts[2]*xs[2])/10 + 
 (epss[3]*ftinv[5]*gs[1]*ts[2]*xs[2])/20 - (deltas[1]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[3])/6 + (deltas[1]*epss[3]*ftinv[5]*gs[1]*ts[1]*xs[3])/10 - 
 (deltas[1]*epss[3]*ftinv[6]*gs[1]*ts[3]*xs[3])/3 + (ftinv[3]*gs[1]*xs[4])/24 - (epss[2]*ftinv[4]*gs[1]*xs[4])/96 + (epss[4]*ftinv[5]*gs[1]*xs[4])/320 - 
 (deltas[2]*epss[2]*ftinv[6]*gs[2]*xs[4])/480 - (epss[3]*ftinv[6]*gs[2]*xs[4])/480 + (epss[2]*ftinv[5]*gs[1]*ts[2]*xs[4])/12 - 
 (epss[4]*ftinv[6]*gs[1]*ts[2]*xs[4])/16 + (epss[4]*ftinv[7]*gs[1]*ts[4]*xs[4])/8 + (deltas[1]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[5])/360 - 
 (epss[2]*ftinv[6]*gs[2]*xs[6])/864 + (epss[4]*ftinv[7]*gs[2]*xs[6])/2880 - (epss[4]*ftinv[8]*gs[2]*ts[2]*xs[6])/288 + (epss[4]*ftinv[9]*gs[3]*xs[8])/17280;
}
#endif

#if _p == 6
double Vp(void)
{
   xs[5] = xs[4] * xs[1];
   xs[6] = xs[5] * xs[1];
   xs[7] = xs[6] * xs[1];
   xs[8] = xs[7] * xs[1];
   gs[2] = gs[1] * gs[1];
   gs[3] = gs[2] * gs[1];
   ts[2] = ts[1] * ts[1];
   ts[3] = ts[2] * ts[1];
   ts[4] = ts[3] * ts[1];
   ftinv[4] = ftinv[3] * ftinv[1];
   ftinv[5] = ftinv[4] * ftinv[1];
   ftinv[6] = ftinv[5] * ftinv[1];
   ftinv[7] = ftinv[6] * ftinv[1];
   ftinv[8] = ftinv[7] * ftinv[1];
   ftinv[9] = ftinv[8] * ftinv[1];
   return (deltas[4]*ftinv[3]*gs[1])/120 + (deltas[2]*epss[1]*ftinv[3]*gs[1])/120 + (epss[2]*ftinv[3]*gs[1])/240 - (deltas[4]*epss[2]*ftinv[4]*gs[1])/224 - 
 (3*deltas[2]*epss[3]*ftinv[4]*gs[1])/1120 - (epss[4]*ftinv[4]*gs[1])/2240 - (deltas[6]*epss[2]*ftinv[6]*gs[2])/64800 - 
 (deltas[4]*epss[3]*ftinv[6]*gs[2])/21600 - (13*deltas[2]*epss[4]*ftinv[6]*gs[2])/201600 - (47*epss[5]*ftinv[6]*gs[2])/1209600 + 
 (deltas[4]*epss[2]*ftinv[5]*gs[1]*ts[2])/28 + (3*deltas[2]*epss[3]*ftinv[5]*gs[1]*ts[2])/140 + (epss[4]*ftinv[5]*gs[1]*ts[2])/280 - 
 (deltas[3]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[1])/10 - (deltas[1]*epss[2]*ftinv[4]*gs[1]*ts[1]*xs[1])/20 + 
 (deltas[3]*epss[3]*ftinv[5]*gs[1]*ts[1]*xs[1])/14 + (3*deltas[1]*epss[4]*ftinv[5]*gs[1]*ts[1]*xs[1])/140 - 
 (5*deltas[3]*epss[3]*ftinv[6]*gs[1]*ts[3]*xs[1])/21 - (deltas[1]*epss[4]*ftinv[6]*gs[1]*ts[3]*xs[1])/14 + (deltas[2]*ftinv[3]*gs[1]*xs[2])/12 + 
 (epss[1]*ftinv[3]*gs[1]*xs[2])/24 - (3*deltas[2]*epss[2]*ftinv[4]*gs[1]*xs[2])/80 - (epss[3]*ftinv[4]*gs[1]*xs[2])/160 + 
 (3*deltas[2]*epss[4]*ftinv[5]*gs[1]*xs[2])/224 + (3*epss[5]*ftinv[5]*gs[1]*xs[2])/2240 - (deltas[4]*epss[2]*ftinv[6]*gs[2]*xs[2])/1440 - 
 (deltas[2]*epss[3]*ftinv[6]*gs[2]*xs[2])/720 - (epss[4]*ftinv[6]*gs[2]*xs[2])/1152 + (3*deltas[2]*epss[2]*ftinv[5]*gs[1]*ts[2]*xs[2])/10 + 
 (epss[3]*ftinv[5]*gs[1]*ts[2]*xs[2])/20 - (15*deltas[2]*epss[4]*ftinv[6]*gs[1]*ts[2]*xs[2])/56 - (3*epss[5]*ftinv[6]*gs[1]*ts[2]*xs[2])/112 + 
 (15*deltas[2]*epss[4]*ftinv[7]*gs[1]*ts[4]*xs[2])/28 + (3*epss[5]*ftinv[7]*gs[1]*ts[4]*xs[2])/56 - (deltas[1]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[3])/6 + 
 (deltas[1]*epss[3]*ftinv[5]*gs[1]*ts[1]*xs[3])/10 + (deltas[3]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[3])/360 + 
 (deltas[1]*epss[4]*ftinv[7]*gs[2]*ts[1]*xs[3])/360 - (deltas[1]*epss[3]*ftinv[6]*gs[1]*ts[3]*xs[3])/3 + (ftinv[3]*gs[1]*xs[4])/24 - 
 (epss[2]*ftinv[4]*gs[1]*xs[4])/96 + (epss[4]*ftinv[5]*gs[1]*xs[4])/320 - (deltas[2]*epss[2]*ftinv[6]*gs[2]*xs[4])/480 - 
 (epss[3]*ftinv[6]*gs[2]*xs[4])/480 + (23*deltas[2]*epss[4]*ftinv[7]*gs[2]*xs[4])/20160 + (epss[5]*ftinv[7]*gs[2]*xs[4])/2016 + 
 (epss[2]*ftinv[5]*gs[1]*ts[2]*xs[4])/12 - (epss[4]*ftinv[6]*gs[1]*ts[2]*xs[4])/16 - (31*deltas[2]*epss[4]*ftinv[8]*gs[2]*ts[2]*xs[4])/2016 - 
 (101*epss[5]*ftinv[8]*gs[2]*ts[2]*xs[4])/20160 + (epss[4]*ftinv[7]*gs[1]*ts[4]*xs[4])/8 + (deltas[1]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[5])/360 + 
 (deltas[3]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[5])/2016 + (deltas[1]*epss[4]*ftinv[7]*gs[2]*ts[1]*xs[5])/16128 - (epss[2]*ftinv[6]*gs[2]*xs[6])/864 + 
 (epss[4]*ftinv[7]*gs[2]*xs[6])/2880 + (deltas[2]*epss[4]*ftinv[9]*gs[3]*xs[6])/8640 + (epss[5]*ftinv[9]*gs[3]*xs[6])/6480 - 
 (epss[4]*ftinv[8]*gs[2]*ts[2]*xs[6])/288 + (epss[4]*ftinv[9]*gs[3]*xs[8])/17280;
}
#endif

#if _p == 7
double Vp(void)
{
   xs[5] = xs[4] * xs[1];
   xs[6] = xs[5] * xs[1];
   xs[7] = xs[6] * xs[1];
   xs[8] = xs[7] * xs[1];
   xs[9] = xs[8] * xs[1];
   xs[10] = xs[9] * xs[1];
   gs[2] = gs[1] * gs[1];
   gs[3] = gs[2] * gs[1];
   gs[4] = gs[3] * gs[1];
   ts[2] = ts[1] * ts[1];
   ts[3] = ts[2] * ts[1];
   ts[4] = ts[3] * ts[1];
   ts[5] = ts[4] * ts[1];
   ts[6] = ts[5] * ts[1];
   ftinv[4] = ftinv[3] * ftinv[1];
   ftinv[5] = ftinv[4] * ftinv[1];
   ftinv[6] = ftinv[5] * ftinv[1];
   ftinv[7] = ftinv[6] * ftinv[1];
   ftinv[8] = ftinv[7] * ftinv[1];
   ftinv[9] = ftinv[8] * ftinv[1];
   ftinv[10] = ftinv[9] * ftinv[1];
   ftinv[11] = ftinv[10] * ftinv[1];
   ftinv[12] = ftinv[11] * ftinv[1];
   return (deltas[4]*ftinv[3]*gs[1])/120 + (deltas[2]*epss[1]*ftinv[3]*gs[1])/120 + (epss[2]*ftinv[3]*gs[1])/240 - (deltas[4]*epss[2]*ftinv[4]*gs[1])/224 - 
 (3*deltas[2]*epss[3]*ftinv[4]*gs[1])/1120 - (epss[4]*ftinv[4]*gs[1])/2240 + (deltas[4]*epss[4]*ftinv[5]*gs[1])/576 + 
 (deltas[2]*epss[5]*ftinv[5]*gs[1])/1344 + (epss[6]*ftinv[5]*gs[1])/13440 - (deltas[6]*epss[2]*ftinv[6]*gs[2])/64800 - 
 (deltas[4]*epss[3]*ftinv[6]*gs[2])/21600 - (13*deltas[2]*epss[4]*ftinv[6]*gs[2])/201600 - (47*epss[5]*ftinv[6]*gs[2])/1209600 + 
 (deltas[4]*epss[2]*ftinv[5]*gs[1]*ts[2])/28 + (3*deltas[2]*epss[3]*ftinv[5]*gs[1]*ts[2])/140 + (epss[4]*ftinv[5]*gs[1]*ts[2])/280 - 
 (5*deltas[4]*epss[4]*ftinv[6]*gs[1]*ts[2])/144 - (5*deltas[2]*epss[5]*ftinv[6]*gs[1]*ts[2])/336 - (epss[6]*ftinv[6]*gs[1]*ts[2])/672 + 
 (5*deltas[4]*epss[4]*ftinv[7]*gs[1]*ts[4])/72 + (5*deltas[2]*epss[5]*ftinv[7]*gs[1]*ts[4])/168 + (epss[6]*ftinv[7]*gs[1]*ts[4])/336 - 
 (deltas[3]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[1])/10 - (deltas[1]*epss[2]*ftinv[4]*gs[1]*ts[1]*xs[1])/20 + 
 (deltas[3]*epss[3]*ftinv[5]*gs[1]*ts[1]*xs[1])/14 + (3*deltas[1]*epss[4]*ftinv[5]*gs[1]*ts[1]*xs[1])/140 + 
 (deltas[5]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[1])/2700 + (301*deltas[3]*epss[4]*ftinv[7]*gs[2]*ts[1]*xs[1])/345600 + 
 (1789*deltas[1]*epss[5]*ftinv[7]*gs[2]*ts[1]*xs[1])/3225600 - (5*deltas[3]*epss[3]*ftinv[6]*gs[1]*ts[3]*xs[1])/21 - 
 (deltas[1]*epss[4]*ftinv[6]*gs[1]*ts[3]*xs[1])/14 + (deltas[2]*ftinv[3]*gs[1]*xs[2])/12 + (epss[1]*ftinv[3]*gs[1]*xs[2])/24 - 
 (3*deltas[2]*epss[2]*ftinv[4]*gs[1]*xs[2])/80 - (epss[3]*ftinv[4]*gs[1]*xs[2])/160 + (3*deltas[2]*epss[4]*ftinv[5]*gs[1]*xs[2])/224 + 
 (3*epss[5]*ftinv[5]*gs[1]*xs[2])/2240 - (deltas[4]*epss[2]*ftinv[6]*gs[2]*xs[2])/1440 - (deltas[2]*epss[3]*ftinv[6]*gs[2]*xs[2])/720 - 
 (epss[4]*ftinv[6]*gs[2]*xs[2])/1152 + (151*deltas[4]*epss[4]*ftinv[7]*gs[2]*xs[2])/302400 + (deltas[2]*epss[5]*ftinv[7]*gs[2]*xs[2])/1575 + 
 (67*epss[6]*ftinv[7]*gs[2]*xs[2])/403200 + (3*deltas[2]*epss[2]*ftinv[5]*gs[1]*ts[2]*xs[2])/10 + (epss[3]*ftinv[5]*gs[1]*ts[2]*xs[2])/20 - 
 (15*deltas[2]*epss[4]*ftinv[6]*gs[1]*ts[2]*xs[2])/56 - (3*epss[5]*ftinv[6]*gs[1]*ts[2]*xs[2])/112 - 
 (1003*deltas[4]*epss[4]*ftinv[8]*gs[2]*ts[2]*xs[2])/151200 - (403*deltas[2]*epss[5]*ftinv[8]*gs[2]*ts[2]*xs[2])/50400 - 
 (49*epss[6]*ftinv[8]*gs[2]*ts[2]*xs[2])/28800 + (15*deltas[2]*epss[4]*ftinv[7]*gs[1]*ts[4]*xs[2])/28 + (3*epss[5]*ftinv[7]*gs[1]*ts[4]*xs[2])/56 - 
 (deltas[1]*epss[1]*ftinv[4]*gs[1]*ts[1]*xs[3])/6 + (deltas[1]*epss[3]*ftinv[5]*gs[1]*ts[1]*xs[3])/10 - 
 (5*deltas[1]*epss[5]*ftinv[6]*gs[1]*ts[1]*xs[3])/112 + (deltas[3]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[3])/360 + 
 (deltas[5]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[3])/1296 + (deltas[1]*epss[4]*ftinv[7]*gs[2]*ts[1]*xs[3])/360 + 
 (827*deltas[3]*epss[4]*ftinv[7]*gs[2]*ts[1]*xs[3])/1451520 + (23*deltas[1]*epss[5]*ftinv[7]*gs[2]*ts[1]*xs[3])/276480 - 
 (deltas[1]*epss[3]*ftinv[6]*gs[1]*ts[3]*xs[3])/3 + (5*deltas[1]*epss[5]*ftinv[7]*gs[1]*ts[3]*xs[3])/14 - 
 (deltas[1]*epss[5]*ftinv[8]*gs[1]*ts[5]*xs[3])/2 + (ftinv[3]*gs[1]*xs[4])/24 - (epss[2]*ftinv[4]*gs[1]*xs[4])/96 + (epss[4]*ftinv[5]*gs[1]*xs[4])/320 - 
 (5*epss[6]*ftinv[6]*gs[1]*xs[4])/5376 - (deltas[2]*epss[2]*ftinv[6]*gs[2]*xs[4])/480 - (epss[3]*ftinv[6]*gs[2]*xs[4])/480 + 
 (23*deltas[2]*epss[4]*ftinv[7]*gs[2]*xs[4])/20160 + (epss[5]*ftinv[7]*gs[2]*xs[4])/2016 + (deltas[4]*epss[4]*ftinv[9]*gs[3]*xs[4])/16200 + 
 (7*deltas[2]*epss[5]*ftinv[9]*gs[3]*xs[4])/43200 + (299*epss[6]*ftinv[9]*gs[3]*xs[4])/2419200 + (epss[2]*ftinv[5]*gs[1]*ts[2]*xs[4])/12 - 
 (epss[4]*ftinv[6]*gs[1]*ts[2]*xs[4])/16 + (15*epss[6]*ftinv[7]*gs[1]*ts[2]*xs[4])/448 - (31*deltas[2]*epss[4]*ftinv[8]*gs[2]*ts[2]*xs[4])/2016 - 
 (101*epss[5]*ftinv[8]*gs[2]*ts[2]*xs[4])/20160 + (epss[4]*ftinv[7]*gs[1]*ts[4]*xs[4])/8 - (5*epss[6]*ftinv[8]*gs[1]*ts[4]*xs[4])/32 + 
 (epss[6]*ftinv[9]*gs[1]*ts[6]*xs[4])/6 + (deltas[1]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[5])/360 + (deltas[3]*epss[3]*ftinv[7]*gs[2]*ts[1]*xs[5])/2016 + 
 (deltas[1]*epss[4]*ftinv[7]*gs[2]*ts[1]*xs[5])/16128 - (deltas[1]*epss[5]*ftinv[8]*gs[2]*ts[1]*xs[5])/560 + 
 (11*deltas[1]*epss[5]*ftinv[9]*gs[2]*ts[3]*xs[5])/1260 - (epss[2]*ftinv[6]*gs[2]*xs[6])/864 + (epss[4]*ftinv[7]*gs[2]*xs[6])/2880 - 
 (17*epss[6]*ftinv[8]*gs[2]*xs[6])/161280 + (deltas[2]*epss[4]*ftinv[9]*gs[3]*xs[6])/8640 + (epss[5]*ftinv[9]*gs[3]*xs[6])/6480 - 
 (epss[4]*ftinv[8]*gs[2]*ts[2]*xs[6])/288 + (13*epss[6]*ftinv[9]*gs[2]*ts[2]*xs[6])/5040 - (epss[6]*ftinv[10]*gs[2]*ts[4]*xs[6])/144 - 
 (deltas[1]*epss[5]*ftinv[10]*gs[3]*ts[1]*xs[7])/8064 + (epss[4]*ftinv[9]*gs[3]*xs[8])/17280 - (11*epss[6]*ftinv[10]*gs[3]*xs[8])/483840 + 
 (epss[6]*ftinv[11]*gs[3]*ts[2]*xs[8])/4480 - (epss[6]*ftinv[12]*gs[4]*xs[10])/276480;
}
#endif
